
/*
 *
 *  Copyright (C) 1996, OFFIS
 *
 *  This software and supporting documentation were developed by
 *
 *    Kuratorium OFFIS e.V.
 *    Forschungsbereich 2: Kommunikationssysteme
 *    Escherweg 2
 *    D-26121 Oldenburg, Germany
 *
 *  for CEN/TC251/WG4 as a contribution to the Computer Assisted Radiology
 *  (CAR) 1996 DICOM Demonstration.
 *
 *  THIS SOFTWARE IS MADE AVAILABLE,  AS IS,  AND OFFIS MAKES NO  WARRANTY
 *  REGARDING  THE  SOFTWARE,  ITS  PERFORMANCE,  ITS  MERCHANTABILITY  OR
 *  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER DISEASES  OR
 *  ITS CONFORMITY TO ANY SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND
 *  PERFORMANCE OF THE SOFTWARE IS WITH THE USER.
 *
 *  Copyright of the software  and  supporting  documentation  is,  unless
 *  otherwise stated, owned by OFFIS, and free access is hereby granted as
 *  a license to  use  this  software,  copy  this  software  and  prepare
 *  derivative works based upon this software.  However, any  distribution
 *  of this software source code or supporting documentation or derivative
 *  works  (source code and  supporting documentation)  must  include  the
 *  three paragraphs of this copyright notice.
 *
 *
 * Module: DICOM Basic Worklist Management CTN, WWW Component
 *
 * Author: Marco Eichelberg
 *
 * Purpose:
 *   Program to read a DICOM worklist file and pass (nearly) all tags
 *   to a WWW CGI perl script (as hexadecimal encoded ASCII via stdout).
 *
 * Last Update:      $Author: meichel $
 * Update Date:      $Date: 1997/05/29 17:06:44 $
 * Source File:      $Source: /share/dicom/cvs-depot/dcmtk/wlistctn/wwwapps/readwlst.cc,v $
 * CVS/RCS Revision: $Revision: 1.5 $
 * Status:           $State: Exp $
 *
 * CVS/RCS Log
 *   $Log: readwlst.cc,v $
 *   Revision 1.5  1997/05/29 17:06:44  meichel
 *   All dcmtk applications now contain a version string
 *   which is displayed with the command line options ("usage" message)
 *   and which can be queried in the binary with the "ident" command.
 *
 *   Revision 1.4  1997/05/22 13:39:37  hewett
 *   Modified the test for presence of a data dictionary to use the
 *   method DcmDataDictionary::isDictionaryLoaded().
 *
 *   Revision 1.3  1997/05/16 08:35:12  andreas
 *   - Revised handling of GroupLength elements and support of
 *     DataSetTrailingPadding elements. The enumeratio E_GrpLenEncoding
 *     got additional enumeration values (for a description see dctypes.h).
 *     addGroupLength and removeGroupLength methods are replaced by
 *     computeGroupLengthAndPadding. To support Padding, the parameters of
 *     element and sequence write functions changed.
 *
 *   Revision 1.2  1997/04/18 08:40:48  andreas
 *   - The put/get-methods for all VRs did not conform to the C++-Standard
 *     draft. Some Compilers (e.g. SUN-C++ Compiler, Metroworks
 *     CodeWarrier, etc.) create many warnings concerning the hiding of
 *     overloaded get methods in all derived classes of DcmElement.
 *     So the interface of all value representation classes in the
 *     library are changed rapidly, e.g.
 *     E_Condition get(Uint16 & value, const unsigned long pos);
 *     becomes
 *     E_Condition getUint16(Uint16 & value, const unsigned long pos);
 *     All (retired) "returntype get(...)" methods are deleted.
 *     For more information see dcmdata/include/dcelem.h
 *
 *   Revision 1.1  1996/05/31 11:43:35  meichel
 *   Initial release of the C/C++ code for the Worklist CTN WWW/CGI application.
 *
 *
 */

#include "osconfig.h"    /* make sure OS specific configuration is included first */

#include <stdio.h>
#include "dctk.h"
#include "dcuid.h"    /* for dcmtk version name */

static char rcsid[] = "$dcmtk: readwlst v"
  OFFIS_DCMTK_VERSION " " OFFIS_DCMTK_RELEASEDATE " $";

void printHexString(char *c)
{
  if (c) while (*c) printf("%02X",(unsigned char)(*c++));
}

void printStringElement(DcmObject& obj, DcmTagKey searchtag)
{
  DcmStack stack;
  char *c = NULL;
  if (obj.search(searchtag, stack, ESM_fromHere, FALSE ) == EC_Normal )
  {
    DcmObject *searchedObj = stack.top();
//    searchedObj->verify(TRUE);
    if (searchedObj->getLength() > 0) switch(searchedObj->ident())
    {
      case EVR_AE:
      case EVR_AS:
      case EVR_CS:
      case EVR_DA:
      case EVR_DS:
      case EVR_DT:
      case EVR_IS:
      case EVR_TM:
      case EVR_UI:
        ((DcmByteString*)searchedObj)->getString(c);
        break;
      case EVR_LO:
      case EVR_LT:
      case EVR_PN:
      case EVR_SH:
      case EVR_ST:
        ((DcmCharString*)searchedObj)->getString(c);
        break;
      default:
        break; 
    }
  }  
  printHexString(c);
}

int main(int argc, char *argv[])
{
    if (argc != 2)
    {
      fprintf(stderr, "%s\n\nusage: readwlst <dicomfile>\n", rcsid);
      return 0;
    }

    /* make sure data dictionary is loaded */
    if (!dcmDataDict.isDictionaryLoaded()) 
    {
      fprintf(stderr, "Warning: no data dictionary loaded, check environment variable: %s\n",
        DCM_DICT_ENVIRONMENT_VARIABLE);
      return 0;
    }

    DcmFileStream myin(argv[1], DCM_ReadMode);
    if ( myin.GetError() != EC_Normal )
    {
        fprintf(stderr, "readwlst: cannot open file: %s\n", argv[1]);
        return 0;
    }

    DcmFileFormat dfile;
    dfile.transferInit();
    dfile.read(myin, EXS_Unknown, EGL_withoutGL);
    dfile.transferEnd();

    if (dfile.error() != EC_Normal)
    {
	fprintf(stderr, "readwlst: error: %s: reading file: %s\n", 
		dcmErrorConditionToString(dfile.error()), argv[1]);
    }

    DcmDataset *dataset = dfile.getDataset();
    if (dataset)
    {
      printf("\\"); printStringElement(*dataset, DCM_RequestedProcedureID);
      printf("\\"); printStringElement(*dataset, DCM_RequestedProcedureDescription);
      printf("\\"); printStringElement(*dataset, DCM_StudyInstanceUID);
      printf("\\"); printStringElement(*dataset, DCM_RequestedProcedurePriority);
      printf("\\"); printStringElement(*dataset, DCM_AccessionNumber);
      printf("\\"); printStringElement(*dataset, DCM_RequestingPhysician);
      printf("\\"); printStringElement(*dataset, DCM_PatientName);
      printf("\\"); printStringElement(*dataset, DCM_PatientID);
      printf("\\"); printStringElement(*dataset, DCM_PatientBirthDate);
      printf("\\"); printStringElement(*dataset, DCM_PatientSex);
      printf("\\"); printStringElement(*dataset, DCM_MedicalAlerts);
      printf("\\"); printStringElement(*dataset, DCM_ContrastAllergies);
      DcmStack stack;
      if (dataset->search(DCM_ScheduledProcedureStepSequence, stack, ESM_fromHere, FALSE ) == EC_Normal )
      {
        DcmSequenceOfItems *seq = (DcmSequenceOfItems *)stack.top();
        if (seq->card() > 0)
        {
          DcmItem *item=seq->getItem(0);
          printf("\\"); if (item) printStringElement(*item, DCM_ScheduledStationAETitle);
          printf("\\"); if (item) printStringElement(*item, DCM_ScheduledProcedureStepStartDate);
          printf("\\"); if (item) printStringElement(*item, DCM_ScheduledProcedureStepStartTime);
          printf("\\"); if (item) printStringElement(*item, DCM_Modality);
          printf("\\"); if (item) printStringElement(*item, DCM_ScheduledPerformingPhysiciansName);
          printf("\\"); if (item) printStringElement(*item, DCM_ScheduledProcedureStepDescription);
          printf("\\"); if (item) printStringElement(*item, DCM_ScheduledStationName);
          printf("\\"); if (item) printStringElement(*item, DCM_ScheduledProcedureStepLocation);
          printf("\\"); if (item) printStringElement(*item, DCM_PreMedication);
          printf("\\"); if (item) printStringElement(*item, DCM_ScheduledProcedureStepID);
          printf("\\"); if (item) printStringElement(*item, DCM_RequestedContrastAgent);
          printf("\\"); if (item) printStringElement(*item, DCM_CommentsOnTheScheduledProcedureStep);
        } else {
          printf("\\\\\\\\\\\\\\\\\\\\\\\\");
        }
      } else {
        printf("\\\\\\\\\\\\\\\\\\\\\\\\");
      }
      printf("\\");
    } else {
      printf("\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\");
    }
}

